@@ -61,7 +61,6 @@ my $track_ignore_list;
 my @track_ignore_list;
 my $outfile_specified = 0;
 
-my $tuplet_factor = 1;
 my $progress_length = 30;
 
 my $quiet = 0;
@@ -79,11 +78,6 @@ while (my $arg = shift) {
         die "missing output file after $arg\n" unless defined $mg_file;
         $outfile_specified = 1;
     }
-    elsif ($arg =~ /^(-t|--tuplet-factor)$/) {
-        $tuplet_factor = shift;
-        die "bad or missing tuplet factor `$tuplet_factor'\n"
-            unless $tuplet_factor > 0;
-    }
     elsif ($arg =~ /^(-w|--ignore-wrong-track)$/) {
         $ignore_wrong_track = 1;
     }
@@ -142,10 +136,6 @@ options:
        track numbers starting from 1.
    -N or --exclude-tracks i[,j...]
        As `-n' but excludes the listed tracks
-   -t n or --tuplet-factor n
-       An additional factor for files with odd note lengths,
-       to prevent them being translated as decimals (n should
-       be a prime number >3).
    -F or --no-factorise
        Do not factorise time values.
 EOF
@@ -245,8 +235,10 @@ foreach my $track (@tracks) {
             $track_name = $event->[2];
             next;
         }
-                
-        my @event_tokens = split ' ', &get_event_string($event);
+
+		my $tokens = &get_event_string($event);
+		next unless defined $tokens;
+        my @event_tokens = split ' ', $tokens;
         @track_tokens = (@track_tokens, @event_tokens)
             unless $event_tokens[0] eq '$ctrl' and
                $event_tokens[1] eq '121,0' and
@@ -463,8 +455,7 @@ sub get_event_string {
         }
         $patch++;
         if ($dtime > 0) {
-            return sprintf "/l%s/r \$patch $patch",
-            &clicks_to_time($dtime);
+            return sprintf "/l%s/r \$patch $patch", &clicks_to_time($dtime);
         }
         else {
             return "\$patch $patch";
@@ -480,8 +471,7 @@ sub get_event_string {
         $value++ if $value == 8191;
         $value = int((8192.5 + $value) / 128);
         if ($dtime > 0) {
-            return sprintf "/l%s/r \$pitch $value",
-            &clicks_to_time($dtime);
+            return sprintf "/l%s/r \$pitch $value", &clicks_to_time($dtime);
         }
         else {
             return "\$pitch $value";
@@ -491,7 +481,7 @@ sub get_event_string {
         my (undef, $dtime, $tempo) = @$event;
         if ($dtime > 0) {
             return sprintf "/l%s/r \$tempo %s",
-            &clicks_to_time($dtime), int(60000000 / $tempo);
+                           &clicks_to_time($dtime), int(60000000 / $tempo);
         }
         else {
             return sprintf "\$tempo %s", int(60000000 / $tempo);
@@ -501,7 +491,7 @@ sub get_event_string {
         my (undef, $dtime, $num, $den) = @$event;
         if ($dtime > 0) {
             return sprintf "/l%s/r \$time_sig $num/%s",
-            &clicks_to_time($dtime), $den * $den;
+                           &clicks_to_time($dtime), $den * $den;
         }
         else {
             return sprintf "\$time_sig $num/%s", $den * $den;
@@ -520,6 +510,7 @@ sub get_event_string {
     }
     elsif ($event->[0] eq 'text_event') {
         my (undef, $dtime, $text) = @$event;
+		return undef if length $text == 0;
 
         if ($dtime > 0) {
             return sprintf "/l%s/r \$text $text", &clicks_to_time($dtime);
@@ -572,7 +563,7 @@ sub get_event_string {
         }
         if ($dtime > 0) {
             return sprintf "/l%s/r \$ctrl $ctrl,$value",
-                &clicks_to_time($dtime);
+                           &clicks_to_time($dtime);
         }
         else {
             return "\$ctrl $ctrl,$value";
@@ -580,10 +571,11 @@ sub get_event_string {
     }
     elsif ($event->[0] eq 'marker') {
         my (undef, $dtime, $text) = @$event;
+		return undef if length $text == 0;
 
         if ($dtime > 0) {
             return sprintf "/l%s/r \$marker \"$text\"",
-                &clicks_to_time($dtime);
+                           &clicks_to_time($dtime);
         }
         else {
             return "\$marker \"$text\"";
@@ -1171,54 +1163,66 @@ sub int_to_note {
 ##
 
 sub clicks_to_time {
-    my $clicks = shift; # time in clicks
-    my $time;           # n:d value to return
-    my $res = 960;      # resolution to work to (n => (1/n)th note)
+    my $clicks = $_[0]; # time in clicks
 
-    $res *= $tuplet_factor;
-    $time = ($clicks * $res) / (4 * $resolution);
+    my $whole = 4 * $resolution;
+	my $hcf = &get_hcf($clicks, $whole);
 
-    if ($res / $time == int($res / $time)) {
-        $time = $res / $time;
-    }
-    else {
-        if ($ctt_do_factorise) {
-            foreach my $i (2, 3, 5, 7, 11, 13, 17, 19) {
-                while (($time / $i == int($time / $i))
-                       and ($res / $i == int($res / $i))) {
-                
-                    $time /= $i;
-                    $res /= $i;
-                }
-            }
-        }
+	$clicks /= $hcf;
+	$whole /= $hcf;
 
-        # try to deal with fractions
-        unless ($time == int $time) {
-            my $factor = 1 / ($time - int $time);
-
-            if ($factor > 1 and $factor == int $factor) {
-                $time *= $factor;
-                $res *= $factor;
-            }
-            else {
-                if ($factor - 3 < .0000001 and
-                    3 - $factor < .0000001) {
-
-                    $time = int(3 * time);
-                    $res *= 3;
-                }
-                elsif (($factor * 2) - 3 < .0000001 and
-                       3 - ($factor * 2) < .0000001) {
-
-                    $time = int(3 * $time);
-                    $res = int(3 * $res);
-                }
-            }
-        }
-
-        $time = "$time:$res";
-    }
-
-    return $time;
+	return ($clicks == 1)? $whole : "$clicks:$whole";
+}
+
+##
+## Return the highest common factor of the two integer arguments.
+##
+
+sub get_hcf {
+	my ($v1, $v2) = @_;
+
+	die "Error: get_hcf(): both arguments must be integers"
+		unless $v1 == int $v1 and $v2 == int $v2;
+	
+	my $f1 = &get_factors($v1);
+	my $f2 = &get_factors($v2);
+
+	my $hcf = 1;
+	my @fa;
+	
+	foreach my $f (@$f1) {
+		shift @$f2 while @$f2 > 0 and $f2->[0] < $f;
+
+		if (@$f2 > 0 and $f2->[0] == $f) {
+			$hcf *= $f;
+			shift @$f2
+		}
+	}
+
+	return $hcf;
+}
+
+##
+## Return the prime factors of the integer argument.
+##
+
+sub get_factors {
+	my $v = shift;
+	die "Error: get_factors(): argument must be an integer\n"
+		unless $v == int $v;
+
+	my @factors;
+
+	for my $prime (qw(2 3 5 7 11 13 17 19 23 29 31)) {
+		last if $v == 1;
+
+		while ($v / $prime == int($v / $prime)) {
+			last if $v == 1;
+			$v /= $prime;
+			push @factors, $prime;
+		}
+	}
+
+	push @factors, $v unless $v == 1;
+	return \@factors;
 }
